/******************************************************************

 Copyright 2001 by Object Craft P/L, Melbourne, Australia.

 LICENCE - see LICENCE file distributed with this software for details.

******************************************************************/

#include "Python.h"
#include "structmember.h"

#define FORCE_PROTOTYPES
#ifdef _WIN32
# define DBNTWIN32
# include <windows.h>
# include "sqlfront.h"
# include "sqldb.h"
# define SQLMAXNAME MAXNAME
# define DBBOOL BOOL
#else
# include "sybfront.h"
# include "sybdb.h"
# define SMALLDATETIBIND SMALLDATETIMEBIND
# define FLT4BIND REALBIND
# define DBFLT4 DBREAL
# define DBDATETIM4 DBDATETIME4
# ifdef MAXNAME
#  define SQLMAXNAME MAXNAME
# else
#  define SQLMAXNAME DBMAXNAME
# endif
# define SQLVOID      SYBVOID
# define SQLTEXT      SYBTEXT
# define SQLVARBINARY SYBVARBINARY
# define SQLINTN      SYBINTN
# define SQLVARCHAR   SYBVARCHAR
# define SQLBINARY    SYBBINARY
# define SQLIMAGE     SYBIMAGE
# define SQLCHAR      SYBCHAR
# define SQLINT1      SYBINT1
# define SQLBIT       SYBBIT
# define SQLINT2      SYBINT2
# define SQLINT4      SYBINT4
# define SQLMONEY     SYBMONEY
# define SQLDATETIME  SYBDATETIME
# define SQLFLT8      SYBFLT8
# define SQLFLTN      SYBFLTN
# define SQLMONEYN    SYBMONEYN
# define SQLDATETIMN  SYBDATETIMN
# define SQLFLT4      SYBREAL
# define SQLMONEY4    SYBMONEY4
# define SQLDATETIM4  SYBDATETIME4
# define SQLDECIMAL   SYBDECIMAL
# define SQLNUMERIC   SYBNUMERIC
#endif

#ifdef HAVE_FREETDS
typedef unsigned char DBBIT;
#define PRNUMERIC MAXPRECISION
#define DB_MAX_PREC MAXPRECISION
#define DB_MAX_SCALE MAXPRECISION
#endif

enum { TYPE_RETCODE, TYPE_ANSWER, TYPE_ROWRES, TYPE_VALUE,
       TYPE_SERVTYPE, TYPE_RPCOPT, TYPE_RPCPARAM, TYPE_BIND,
       TYPE_OPTION,
};

void debug_msg(char *fmt, ...);
char *value_str(int type, int value);

typedef struct {
    PyObject_HEAD

    LOGINREC *login;
    int debug;
} LOGINRECObj;

extern PyTypeObject LOGINRECType;
PyObject *LOGINREC__init__(int debug);

typedef struct DBPROCObj {
    PyObject_HEAD

    DBPROCESS *dbproc;
    int debug;
    struct DBPROCObj *next;
} DBPROCObj;

extern PyTypeObject DBPROCType;
PyObject *dbproc_find_object(DBPROCESS *db_dbproc);
PyObject *DBPROC__init__(LOGINREC *login, char *server, int debug);

typedef struct {
    PyObject_HEAD
    int strip;
    int vartype;
    DBINT varlen;
    DBINT size;
    BYTE *buff;
} DataBufObj;

extern PyTypeObject DataBufType;
#define DataBuf_Check(obj) (obj->ob_type == &DataBufType)
int is_fixedlen(int vartype);
PyObject *DataBuf_convert(DBPROCObj *dbproc, PyObject *from, DataBufObj *to);
PyObject *DataBuf__init__(int vartype, int varlen);

typedef struct {
    PyObject_HEAD
    DBNUMERIC num;
} NumericObj;

#define NUMERIC_LEN PRNUMERIC

extern PyTypeObject NumericType;
#define Numeric_Check(obj) (obj->ob_type == &NumericType)
PyObject *numeric_alloc(DBNUMERIC *num);
int numeric_as_string(PyObject *obj, char *text);
extern char numeric_new__doc__[];
PyObject *NumericType_new(PyObject *module, PyObject *args);
void copy_reg_numeric(PyObject *dict);
extern char pickle_numeric__doc__[];
PyObject *pickle_numeric(PyObject *module, PyObject *args);

typedef struct {
    PyObject_HEAD
    DBDATETIME datetime;
    DBDATEREC daterec;
    int cracked;
} DateTimeObj;

#define DATETIME_LEN 32

extern PyTypeObject DateTimeType;
#define DateTime_Check(obj) (obj->ob_type == &DateTimeType)
PyObject *datetime_alloc(DBDATETIME *num);
int datetime_as_string(PyObject *obj, char *text);
extern char datetime_new__doc__[];
PyObject *DateTimeType_new(PyObject *module, PyObject *args);
void copy_reg_datetime(PyObject *dict);
extern char pickle_datetime__doc__[];
PyObject *pickle_datetime(PyObject *module, PyObject *args);

typedef struct {
    PyObject_HEAD
    DBMONEY money;
} MoneyObj;

#define MONEY_LEN 32

extern PyTypeObject MoneyType;
#define Money_Check(obj) (obj->ob_type == &MoneyType)
PyObject *money_alloc(DBMONEY *num);
int money_as_string(PyObject *obj, char *text);
extern char money_new__doc__[];
PyObject *MoneyType_new(PyObject *module, PyObject *args);
void copy_reg_money(PyObject *dict);
extern char pickle_money__doc__[];
PyObject *pickle_money(PyObject *module, PyObject *args);
